@import "../node_modules/@angular/material/theming";
@include mat-core();

$primary: mat-palette($mat-purple);
$accent: mat-palette($mat-cyan, 500, 900, A100);
$warn: mat-palette($mat-red);

$theme: mat-dark-theme($primary, $accent, $warn);
@include angular-material-theme($theme);

.dark {
  $dark-primary: mat-palette($mat-purple);
  $dark-accent:  mat-palette($mat-cyan, 500, 900, A100);
  $dark-theme: mat-dark-theme($dark-primary, $dark-accent);

  @include angular-material-theme($dark-theme);
}

.light {
  $light-primary: mat-palette($mat-purple);
  $light-accent:  mat-palette($mat-cyan, 500, 900, A100);
  $light-theme: mat-light-theme($light-primary, $light-accent);

  @include angular-material-theme($light-theme);
}

