/* You can add global styles to this file, and also import other style files */
$material-icons-font-path: '~material-icons/iconfont/';
@import '~material-icons/iconfont/material-icons.scss';

html,
body {
  height: 100%;
}

body {
  margin: 0;
  background-color: #202020;
}

@font-face {
  font-family: 'Permanent Marker';
  src: url(assets/fonts/PermanentMarker-Regular.ttf) format('opentype');
}

.link-button:hover {
  color: #D0D0D0;
}


.remote-button:hover {
  color: #303030;
}

mat-icon {
  vertical-align: middle;
}

