mat-form-field {
  font-size: 14px;
  width: 90vw;
}

a {
  color: #21a6b8;
}

a:visited {
  color: #cf8313;
}

table tr.mat-row {
  height: 50px;
  overflow: hidden;
}

table {
  text-align: left;
}

.matToolTipClass {
  color: white;
  opacity: 1;
  background-color: #303030;
}
