.card-container {
  width: 95vw;
  height: 72vh;
  margin: 2vh auto;
  min-width: 1500px;
  min-height: 700px;
}

#homepage:link, #homepage:visited {
  color: white;
  text-decoration: none;
}

#homepage:hover {
  color: #ADADAD;
}

#homepage:active {
  color: white;
  text-decoration: none;
}

#banner {
  font-family: 'Permanent Marker';
  font-size: 42px;
  text-shadow: 3px 3px gray;
}

#paypal {
  margin-right: 1em;
}

#paypal:hover {
  background-color: #002c8b;
}

