<!DOCTYPE html>
<html lang="en">

<head>
  <meta charset="utf-8" />
  <title>MagicMirror Package Manager</title>
  <base href="/" />
  <meta name="viewport" content="width=device-width, initial-scale=1" />
  <link rel="icon" type="image/x-icon" href="favicon.ico" />
<link rel="stylesheet" href="static/styles.css"></head>

<body class="mat-typography">
  <app-root></app-root>
<script src="static/runtime-es2015.js" type="module"></script><script src="static/runtime-es5.js" nomodule defer></script><script src="static/polyfills-es5.js" nomodule defer></script><script src="static/polyfills-es2015.js" type="module"></script><script src="static/main-es2015.js" type="module"></script><script src="static/main-es5.js" nomodule defer></script></body>

</html>
